@@ -11,7 +11,7 @@ module REXML
 
 		attr_accessor :value			# The value of the attribute
 		attr_reader :element			# The element to which this attribute belongs
-		PATTERN = /\s*((?:\w[\w-]*:)?\w[\w-]*)\s*=\s*(["'])(.*?)\2/um
+		PATTERN = /\s*(#{NAME_STR})\s*=\s*(["'])(.*?)\2/um
 
 		# Constructor.
 		#
