@@ -7,9 +7,8 @@ module REXML
 	# class directly.
 	class Child
 		include Node
-		attr_accessor :parent		# The Parent of this object
+		attr_reader :parent		# The Parent of this object
 
-		##
 		# Constructor.  Any inheritors of this class should call super to make
 		# sure this method is called.
 		# @param parent if supplied, the parent of this child will be set to the
@@ -19,7 +18,6 @@ module REXML
 			parent << self unless parent.nil?
 		end
 
-		##
 		# Replaces this object with another object.  Basically, calls
 		# Parent.replace_child
 		# @return self
@@ -28,7 +26,6 @@ module REXML
 			self
 		end
 
-		##
 		# Removes this child from the parent.
 		# @return self
 		def remove
@@ -38,9 +35,22 @@ module REXML
 			self
 		end
 
+		def parent=( other )
+			@parent.delete self if defined? @parent and @parent
+			@parent = other
+		end
+
 		alias :next_sibling :next_sibling_node
 		alias :previous_sibling :previous_sibling_node
 
+		def next_sibling=( other )
+		  parent.insert_after self, other
+		end
+
+		def previous_sibling=(other)
+		  parent.insert_before self, other
+		end
+
 		##
 		# @return the document this child belongs to, or nil if this child
 		# belongs to no document
@@ -56,5 +66,19 @@ module REXML
 
 			to_s
 		end
+
+		def Child.once(*ids)
+			for id in ids
+				module_eval <<-"end_eval"
+					alias_method :__#{id.to_i}__, #{id.inspect}
+					def #{id.id2name}(*args, &block)
+						def self.#{id.id2name}(*args, &block)
+							@__#{id.to_i}__
+						end
+						@__#{id.to_i}__ = __#{id.to_i}__(*args, &block)
+					end
+				end_eval
+			end
+		end
 	end
 end
