@@ -126,6 +126,7 @@ module REXML
 		# @param indent (optional) if given, the starting indent for the lines
 		# in the document.
 		def write( output, indent=-1 )
+			output = Output.new( output, xml_decl.encoding ) if xml_decl.encoding != "UTF-8"
 			@children.each { |node|
 				node.write( output, indent )
 				output << "\n" unless indent<0 or node == @children[-1]
@@ -200,9 +201,11 @@ module REXML
 					when DocType::START_RE
 						self.add( DocType.new( source ) )
 					when XMLDecl::START_RE
-						self.add( XMLDecl.new( source ) )
+						x = XMLDecl.new( source )
+						source.encoding = x.encoding
+						self.add( x )
 					when Instruction::START_RE
-						self.add( Instruction.new( source ) )
+						self.add( Instruction.new(source) )
 					else
 						Element.new( source, self, @context )
 					end
