@@ -70,7 +70,41 @@ module REXML
 			end
 		end
 
-		# UNTESTED
+		# A node-set is converted to a string by returning the string-value of the
+		# node in the node-set that is first in document order. If the node-set is
+		# empty, an empty string is returned.
+		#
+		# A number is converted to a string as follows
+		#
+		# NaN is converted to the string NaN 
+		#
+		# positive zero is converted to the string 0 
+		#
+		# negative zero is converted to the string 0 
+		#
+		# positive infinity is converted to the string Infinity 
+		#
+		# negative infinity is converted to the string -Infinity 
+		#
+		# if the number is an integer, the number is represented in decimal form
+		# as a Number with no decimal point and no leading zeros, preceded by a
+		# minus sign (-) if the number is negative
+		#
+		# otherwise, the number is represented in decimal form as a Number
+		# including a decimal point with at least one digit before the decimal
+		# point and at least one digit after the decimal point, preceded by a
+		# minus sign (-) if the number is negative; there must be no leading zeros
+		# before the decimal point apart possibly from the one required digit
+		# immediately before the decimal point; beyond the one required digit
+		# after the decimal point there must be as many, but only as many, more
+		# digits as are needed to uniquely distinguish the number from all other
+		# IEEE 754 numeric values.
+		#
+		# The boolean false value is converted to the string false. The boolean
+		# true value is converted to the string true.
+		#
+		# An object of a type other than the four basic types is converted to a
+		# string in a way that is dependent on that type.
 		def Functions::string( object=nil )
 			object = context unless object
 			if object.kind_of? Array
@@ -123,7 +157,11 @@ module REXML
 		# UNTESTED
 		def Functions::normalize_space( string=nil )
 			string = string(@@node) if string.nil?
-			string.strip.gsub(/\s+/um, ' ')
+			if string.kind_of? Array
+				string.collect{|x| string.to_s.strip.gsub(/\s+/um, ' ') if string}
+			else
+				string.to_s.strip.gsub(/\s+/um, ' ')
+			end
 		end
 
 		# UNTESTED
@@ -139,15 +177,15 @@ module REXML
 				else
 					return object.size > 0
 				end
-			elsif object.kind_of? Array or object.kind_of? Parent
-				object.size > 0
+			elsif object.kind_of? Array
+				object = object.find{|x| x and true}
 			end
+			return (object and true) & true		# if rv is nil, this turns it into false
 		end
 
 		# UNTESTED
 		def Functions::not( object )
-			return object == "true" if object.is_a? String
-			not object
+			not boolean( object )
 		end
 
 		# UNTESTED
@@ -177,7 +215,31 @@ module REXML
 			lang2.downcase.index(lang1.downcase) == 0
 		end
 
-		def Functions::number( nodes, object=nil )
+		# a string that consists of optional whitespace followed by an optional
+		# minus sign followed by a Number followed by whitespace is converted to
+		# the IEEE 754 number that is nearest (according to the IEEE 754
+		# round-to-nearest rule) to the mathematical value represented by the
+		# string; any other string is converted to NaN
+		#
+		# boolean true is converted to 1; boolean false is converted to 0
+		#
+		# a node-set is first converted to a string as if by a call to the string
+		# function and then converted in the same way as a string argument
+		#
+		# an object of a type other than the four basic types is converted to a
+		# number in a way that is dependent on that type
+		def Functions::number( object=nil )
+			object = @node unless object
+			if object == true
+				object = "1"
+			elsif object == false
+				object = "0"
+			elsif object.kind_of? Array
+				object = string( object )
+			else
+				object = object.to_s
+			end
+			object.to_f
 		end
 
 		def Functions::sum( nodes )
