@@ -5,19 +5,21 @@ module REXML
 		attr_reader :name, :expanded_name
 		# The expanded name of the object, valid if name is set
 		attr_accessor :prefix
+		NCNAME_STR= '[\w_:][-\w\d._]*'
+		NAME_STR= "(?:#{NCNAME_STR}:)?#{NCNAME_STR}"
+		NAMESPLIT = /^(?:(#{NCNAME_STR}):)?(#{NCNAME_STR})/u
 
 		# Sets the name and the expanded name
 		def name=( name )
 			@expanded_name = name
-			if name.include? ":"
-				name =~ /(\w+):(\w+)/u
+			name =~ NAMESPLIT
+			if $1
 				@prefix = $1
-				@name = $2
 			else
-				@name = name
-				@namespace = ""
 				@prefix = ""
+				@namespace = ""
 			end
+			@name = $2
 		end
 
 		# Compares names optionally WITH namespaces
