@@ -21,6 +21,7 @@ module REXML
 			rescue Exception
 				raise "#{$!.message}: only Child-ren can be added to a Parent"
 			end
+			object
 		end
 
 		alias :push :add
@@ -120,7 +121,9 @@ module REXML
 		# @return the index of the child, or nil if the object is not a child
 		# of this parent.
 		def index( child )
-			@children.index child
+			count = -1
+			@children.find { |i| count += 1 ; i.hash == child.hash }
+			count
 		end
 
 		# @return the number of children of this parent
@@ -135,10 +138,8 @@ module REXML
 		def replace_child( to_replace, replacement )
 			ind = @children.index( to_replace )
 			to_replace.parent = nil
-			to_replace.remove
-			@children[ind] = replacement
+			@children[ind,0] = replacement
 			replacement.parent = self
 		end
-
 	end
 end
