@@ -8,11 +8,7 @@ module REXML
 			if arg.kind_of? String
 				source = Source.new(arg)
 			elsif arg.kind_of? IO
-				#if slurp
-				#	source = Source.new( arg.read )
-				#else
-					source = IOSource.new(arg)
-				#end
+				source = IOSource.new(arg)
 			end
 			source
 		end
@@ -25,16 +21,48 @@ module REXML
 		attr_reader :buffer
 		# The line number of the last consumed text
 		attr_reader :line
+		attr_reader :encoding
+
+		UTF16=0
+		UTF8=1
+		ISO_8859_1=2
+		UNILE=3
+		ENCODINGS = {
+			UTF16=>"UTF-16",
+			UTF8=>"UTF-8",
+			ISO_8859_1=>"ISO-8859-1",
+			UNILE=>"UNILE"
+		}
 
 		# Constructor
 		# @param arg must be a String, and should be a valid XML document
 		def initialize arg
 			@orig = @buffer = arg
-			@encoding = encoding( @buffer )
-			@buffer = utf8_enc(@buffer) if (@encoding=='UTF16' or @encoding=="UNILE")
+			@encoding = check_encoding( @buffer )
+			@buffer = utf8_enc(@buffer) if [UTF16,UNILE,ISO_8859_1].include?(@encoding)
 			@line = 0
 		end
 
+		def Source::encoding_val( enc )
+			case enc
+			when /^iso-8859-1$/i; ISO_8859_1
+			when /^utf-8$/i; UTF8
+			when /^unile$/i; UNILE
+			when /^utf-16/i; UTF16
+			end
+		end
+		def Source::val_encoding( enc )
+			ENCODINGS[enc]
+		end
+
+		def encoding=(enc)
+			@encoding = Source::encoding_val(enc)
+			if @encoding != UTF8
+				@to_utf = true
+				@buffer = utf8_enc(@buffer)
+			end
+		end
+
 		# Scans the source for a given pattern.  Note, that this is not your
 		# usual scan() method.  For one thing, the pattern argument has some
 		# requirements; for another, the source can be consumed.  You can easily
@@ -78,42 +106,72 @@ module REXML
 		end
 
 		# Taken from code contributed by Ernest Ellingson <erne@powernav.com>
-		def encoding(str)
+		def check_encoding(str)
 			if str=~/^\376\377/
-				'UTF16'  #unicode big endian
+				UTF16  #unicode big endian
 			elsif str=~/^\377\376/
-				'UNILE' #unicode litle endian
-			elsif str=~/([\300-\377][\200-\277])/  
-				#not a guaranteed test won't detect random pairs
-				'UTF8'
+				UNILE #unicode litle endian
 			else
-				'ASC'
+				UTF8
 			end
 		end
 
 		# Taken from code contributed by Ernest Ellingson <erne@powernav.com>
 		def utf8_enc(str)
-			arrayEnc=[]
-			str.each_byte{|b| arrayEnc << b}
-			arrayUTF8=[]
-			if @encoding=='UTF16'
-				2.step(arrayEnc.size-1, 2){|i| arrayUTF8 << 
-				[arrayEnc.at(i+1) + arrayEnc.at(i) * 0x100].pack("U")}
-			elsif @encoding=='UNILE'
-				2.step(arrayEnc.size-1, 2){|i| arrayUTF8 << 
-				[arrayEnc.at(i) + arrayEnc.at(i+1) * 0x100].pack("U")}
+			if @encoding==ISO_8859_1
+				asc(str)
+			elsif @encoding==UTF16
+				utf16(str)
+			elsif @encoding==UNILE
+				unile(str)
+			else
+				return str
 			end
-			arrayUTF8.join("")
+		end
+
+		def asc(str)
+			str.unpack('C*').pack('U*')
+		end
+		def utf16(str)
+			array_enc=str.unpack('C*')
+			array_utf8 = []
+			2.step(arrayEnc.size-1, 2){|i| 
+				array_utf8 << (array_enc.at(i+1) + array_enc.at(i)*0x100)
+			}
+			array_utf8.pack('U*')
+		end
+		def unile(str)
+			array_enc=str.unpack('C*')
+			array_utf8 = []
+			2.step(array_enc.size-1, 2){|i| 
+				array_utf8 << (array_enc.at(i) + array_enc.at(i+1)*0x100)
+			}
+			array_utf8.pack('U*')
+		end
+		def utf8(str)
+			str
 		end
 	end
 
 	# A Source that wraps an IO.  See the Source class for method
 	# documentation
 	class IOSource < Source
-		def initialize arg
+		attr_reader :block_size
+
+		def initialize arg, block_size=500
 			@er_source = @source = arg
-			super @source.read(500)
-			@to_utf = (@encoding == 'UTF16' or @encoding == "UNILE")
+			@block_size = block_size
+			super @source.read(@block_size)
+			@to_utf = true
+			al = case @encoding
+			when UTF16; :utf16
+			when UNILE; :unile
+			when ISO_8859_1; :asc
+			else
+				@to_utf = false
+				:utf8
+			end
+			instance_eval "alias :encode #{al}"
 		end
 
 		def scan pattern, consume=false
@@ -126,8 +184,8 @@ module REXML
 			if rv.size == 0
 				until @buffer =~ pattern or @source.nil?
 					begin
-						str = @source.read(500)
-						str = utf8_enc(str) if str and @to_utf
+						str = @source.read(@block_size)
+						str = encode(str) if str and @to_utf
 						@buffer << str
 					rescue
 						@source = nil
@@ -137,12 +195,13 @@ module REXML
 			end
 			rv
 		end
+
 		def match pattern, consume=false
 			rv = super
 			while !rv and @source
 				begin
-					str = @source.read(500)
-					str = utf8_enc(str) if str and @to_utf
+					str = @source.read(@block_size)
+					str = encode(str) if str and @to_utf
 					@buffer << str
 					rv = super
 				rescue
@@ -151,8 +210,9 @@ module REXML
 			end
 			rv
 		end
+		
 		def empty?
-			super and ( @source || @source.eof? )
+			super and ( @source.nil? || @source.eof? )
 		end
 
 		# @return the current line in the source
