@@ -1,3 +1,5 @@
+require 'rexml/source'
+
 module REXML
 	# NEEDS DOCUMENTATION
 	class XMLDecl < Child
@@ -13,9 +15,10 @@ module REXML
 		STANDALONE_RE = /\bstandalone=["'](.*?)['"]/um
 		PATTERN = /<\?xml\s+(.*?)\?>*/um
 
-		attr_accessor :version, :standalone, :encoding
+		attr_accessor :version, :standalone
 
 		def initialize(version=DEFAULT_VERSION, encoding=nil, standalone=nil)
+			@encoding = nil
 			if version.kind_of? Source
 				super encoding
 				XMLDecl.parse_stream version, self
@@ -23,12 +26,12 @@ module REXML
 			elsif version.kind_of? XMLDecl
 				super()
 				@version = version.version
-				@encoding = version.encoding
+				self.encoding = version.encoding
 				@standalone = version.standalone
 			else
 				super()
 				@version = version
-				@encoding = encoding
+				self.encoding = encoding
 				@standalone = standalone
 			end
 			@version = DEFAULT_VERSION if @version.nil?
@@ -48,13 +51,13 @@ module REXML
 		def ==( other )
 		  other.kind_of?(XMLDecl) and
 		  other.version == @version and
-		  other.encoding == @encoding and
+		  other.encoding == self.encoding and
 		  other.standalone == @standalone
 		end
 
 		def xmldecl version, encoding, standalone
 			@version = version
-			@encoding = encoding
+			self.encoding = encoding
 			@standalone = standalone
 		end
 
@@ -70,10 +73,17 @@ module REXML
 			listener.xmldecl version, encoding, standalone
 		end
 
+		def encoding=( value )
+			@encoding = Source::encoding_val(value)
+		end
+		def encoding
+			Source.val_encoding @encoding
+		end
+
 		private
 		def content
 			rv = "version='#@version'"
-			rv << " encoding='#@encoding'" if @encoding
+			rv << " encoding='#{encoding}'" if @encoding
 			rv << " standalone='#@standalone'" if @standalone
 			rv
 		end
